;
; Test_SPI.asm
;
; Created: 02.11.2020 00:36:57
; Author : N0082
;
.include	"port_io.inc"
.include	"macros.inc"



.equ	F_CPU				= 16000000

#define	EXT_RESET
.equ	RST_PORT			= PORTB
.equ	RST_PIN				= 1

; OLED Display settings
.equ	Display_Xsize					= 128	// in Pixel	(DOT's)	[1..320]
.equ	Display_Ysize					= 128	// in Pixel (DOT's)	[1..256]

#define	OLED_DISPLAY_ANGEL				0		// 0 , 90 , 180 , 270 degree


;#define		USE_DELAY

.def	ZERO				= r13
.def	ONE					= r14
.def	iSREG				= r15
.def	iTEMPA				= r23

.include	"sram_variables.inc"
.include	"tn1614_INT_vectors.inc"
.include	"delay.inc"
.include	"spi.inc"
.include	"SSD1351_driver.inc"
.include	"gfx128.inc"
.include	"txt_gfx.inc"
RESET:

		clr		ZERO
		ldi		r16,1
		mov		ONE,r16

		ldi		YL,Byte1(1300)
		ldi		YH,Byte2(1300)
		load_p	Z,(INTERNAL_SRAM_START)
flush_sRAM:
		st		Z+,zero
		sbiw	Y,1
		brne	flush_sRAM


		sbi		VPORTA+0,PA7
		sbi		VPORTA+1,PA7


;---- CLK Source -----------------------------
		ldi		r16,(CLKCTRL_CLKSEL_OSC20M_gc);(1<<CLKCTRL_CLKOUT_bp)+
		ldi		r17,(CPU_CCP_IOREG_gc)
		out		(CPU_CCP),r17
		sts		(CLKCTRL_MCLKCTRLA),r16

wait_clk_osc20_stable:
		lds		r16,(CLKCTRL_MCLKSTATUS)
		sbrs	r16,(CLKCTRL_OSC20MS_bp)
		rjmp	wait_clk_osc20_stable

;---- CLK Prescaller -----------------------------
		ldi		r16,(CLKCTRL_PDIV_2X_gc)+(0<<CLKCTRL_PEN_bp)
		ldi		r17,(CPU_CCP_IOREG_gc)
		out		(CPU_CCP),r17
		sts		(CLKCTRL_MCLKCTRLB),r16

#if defined EXT_RESET
;---- RESET PIN ----------------------------------
		ldi		r16,(1<<PORT_PULLUPEN_bp)|(PORT_ISC_FALLING_gc)
		sts		((RST_PORT)+(RST_PIN)+(PORT_PIN0CTRL_offset)),r16			;PORTB_PIN1CTRL
#endif
		rcall	init_spi
		sei

		rcall	oled_init

/*
		ldi16	r16,r17,(WHITE)
		ldi		XL,10
		ldi		YL,10
		ldi		XH,4
		rcall	oled_fill_circle
*/
		rjmp	test_txt	

test_txt:

		draw_fcircle	10,10,4,(WHITE)
		draw_rect		0,0,128,128,(WHITE)
		draw_frdrect	31,109,66,13,4,((2<<RED_bp)|(16<<GREEN_bp)|(10<<BLUE_bp))
		draw_rdrect		30,108,68,15,5,(WHITE)


		set_xypos	24,24
		set_txt_color	(RED)
		set_back_color	(BLACK)
		fld_txt	(apetxt_01)							// Hallo!
		rcall	oled_write_ape_fstring


		set_xypos		52,56			;52
		set_txt_color	(BLUE)
		set_back_color	(BLACK)
		fld_txt	(apetxt_02)							// AVR
		rcall	oled_write_ape_fstring


		set_xypos		50,85
		set_txt_color	(YELLOW)
		set_back_color	(BLACK)
		fld_txt	(apetxt_03)							// ATtiny1614
		rcall	oled_write_ape_fstring

	
		set_xypos		35,112
		set_txt_color	(YELLOW)
		set_back_color	((2<<RED_bp)|(16<<GREEN_bp)|(10<<BLUE_bp))
		fld_txt	(apetxt_04)
		rcall	oled_write_ape_fstring


		lds		r16,(CharPosX)
		subi	r16,(-5)
		sts		(CharPosX),r16
		set_txt_color	(GREEN) | (10<<BLUE_bp)
		set_back_color	((2<<RED_bp)|(16<<GREEN_bp)|(10<<BLUE_bp))
		fld_txt	(apetxt_05)
		rcall	oled_write_ape_fstring


		rjmp	start


test_rotate:
		delay	2000
;		rcall	oled_clr_screen

		lds		r0,(ORIANTATION)
		sbrc	r0,3
		rjmp	reload_rotation
		clc
load_rotation:
		rol		r0
		sts		(ORIANTATION),r0
		rcall	oled_set_oriantation
		rjmp	test_rotate
reload_rotation:
		clr		r0
		sec
		rjmp	load_rotation


; Replace with your application code
start:
		inc		r16
		rjmp	start



;.include	"E:\Atmel\APE_Fonts\Arial_10x16.inc"
.include	"E:\Atmel\APE_Fonts\Comic_Sans_10x32_Bold.inc"
.include	"E:\Atmel\APE_Fonts\FONT_5x8.inc"

; Example to write STRINGS from FLASH
; LABEL:			LABEL of Font					  .db <CharSpace>,"STRING",0 <-- Termierung
;otxt_test:		.dw	(COMIC_SANS_10X32_BOLD <<1)			\ .db	1,	"Test",0

apetxt_01:		.dw	(COMIC_SANS_10X32_BOLD <<1)			\ .db	2,	"Hallo!",0
apetxt_02:		.dw	(COMIC_SANS_10X32_BOLD <<1)			\ .db	1,	"AVR",0
apetxt_03:		.dw	(FONT_5x8 <<1)						\ .db	3,	"ATtiny1614",0
apetxt_04:		.dw	(FONT_5x8 <<1)						\ .db	1,	"Mini",0
apetxt_05:		.dw	(FONT_5x8 <<1)						\ .db	1,	"Schrift",0




/*
 * gfx128.inc
 *
 *  Created: 06.11.2020 18:36:31
 *   Author: N0082
 */ 
 
;#### VVVVV ################ GFX ##### GFX ##### GFX ##### GFX ########################################################################
;######################################################################################################################################

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef	use_fill_screen
oled_fill_screen:	; Fill Screen with Color [r16 = COLOR L]|[r17 = COLOR H]
		clr		XL
		ldi		XH,((Display_Xsize)-1)
		clr		YL
		ldi		YH,((Display_Ysize)-1)
#endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef	use_fill_window
oled_fill_window:
		movw	r2,XL
		sub		r3,r2
		inc		r3
		mov		r4,r3				; r4 Xsize
		movw	r2,YL
		sub		r3,r2
		inc		r3
;		mov		r5,r3				; r3,r5 Ysize
oled_fill_window_start:
		cbi		((SPI0_CS_VPORT)+1),(SPI0_CS)					; VPORT.OUT.CS = 0		--> SPI DEVICE SELECTED
		rcall	oled_set_xy
;SPI write Command
		mov		r1,r18
		cbi		((SSD1351_DC_VPORT)+1),(SSD1351_DC)				; VPORT.OUT.DC = 0		--> (COMMAND)
		ldi		r18,(SSD1351_CMD_WRITERAM)						;  write COMMAND 
		sts		(SPI0_DATA),r18									; "WRITE DATA TO DRAM"
;wait spi->TX->ready
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_TXCIF_bp)
		rjmp	PC -3
		sts		(SPI0_INTFLAGS),r0
		sbi		((SSD1351_DC_VPORT)+1),(SSD1351_DC)				; VPORT.OUT.DC = 1		--> (DATA)

oled_fill_screen_loop_i:
		mov		r5,r3
oled_fill_screen_loop_j:	
;SPI write Data									
		sts		(SPI0_DATA),r17								; write DATA PIX COLOR H
;SPI write Data									
		sts		(SPI0_DATA),r16								; write DATA PIX COLOR L
;wait spi->TX->ready
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_TXCIF_bp)
		rjmp	PC -3
		sts		(SPI0_INTFLAGS),r0
		dec		r5
		brne	oled_fill_screen_loop_j
		dec		r4
		brne	oled_fill_screen_loop_i
		sbi		((SPI0_CS_VPORT)+1),(SPI0_CS)					; VPORT.OUT.CS = 1		--> SPI DEVICE DESELECTED
		mov		r18,r1
		ret
#endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#if defined	use_draw_hline || defined use_draw_line || defined use_draw_rect || defined use_draw_round_rect || defined use_draw_fill_round_rect || defined use_draw_fill_circle
oled_draw_hline:	; XL = Xstart	|	YL = Ystart	|	XH = lenght	|	r17:r16 = COLOR
		ldi		YH,(Display_Xsize)
		cp		XL,YH					; Xstart >= Display_Xsize
		brsh	oled_draw_hline_end		; --> abort
		cpi		YL,(Display_Ysize)		; Ystart >= Display_Ysize
		brsh	oled_draw_hline_end		; --> abort
		mov		r0,XL					; make copy of XL
		add		r0,XH					; (Xstart + length) > Display_Xsize ?
		cp		YH,r0					; Display_Xsize < (Xstart + length)
		brsh	oled_draw_hline_calc
		sub		YH,XL
		dec		YH						; length = Display_Xsize - Xstart -1
		mov		XH,YH					; copy lenght		
oled_draw_hline_calc:
		add		XH,XL
		dec		XH						; XH = Xend
		mov		r18,YH					; r18 = length
		mov		YH,YL					; Ystart = Yend
		cbi		((SPI0_CS_VPORT)+1),(SPI0_CS)					; VPORT.OUT.CS = 0		--> SPI DEVICE SELECTED
		rcall	oled_set_xy
;SPI write Command
		cbi		((SSD1351_DC_VPORT)+1),(SSD1351_DC)				; VPORT.OUT.DC = 0		--> (COMMAND)
		ldi		XH,(SSD1351_CMD_WRITERAM)						;  write COMMAND 
		sts		(SPI0_DATA),XH									; "WRITE DATA TO DRAM"
;wait spi->TX->ready
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_TXCIF_bp)
		rjmp	PC -3
		sts		(SPI0_INTFLAGS),r0
		sbi		((SSD1351_DC_VPORT)+1),(SSD1351_DC)				; VPORT.OUT.DC = 1		--> (DATA)
oled_draw_hline_loop:
;SPI write Data									
		sts		(SPI0_DATA),r17								; write DATA PIX COLOR H
;SPI write Data									
		sts		(SPI0_DATA),r16								; write DATA PIX COLOR L
;wait spi->TX->ready
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_TXCIF_bp)
		rjmp	PC -3
		sts		(SPI0_INTFLAGS),r0
		dec		r18
		brne	oled_draw_hline_loop
		sbi		((SPI0_CS_VPORT)+1),(SPI0_CS)					; VPORT.OUT.CS = 1		--> SPI DEVICE DESELECTED
oled_draw_hline_end:
		ret
#endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#if	defined	use_draw_vline || defined use_draw_line || defined use_draw_rect || defined use_draw_round_rect || defined use_draw_fill_round_rect || defined use_draw_fill_circle
oled_draw_vline:	; XL = Xstart	|	YL = Ystart	|	YH = lenght	|	r17:r16 = COLOR
		ldi		XH,(Display_Ysize)
		cpi		XL,(Display_Xsize)		; Xstart >= Display_Xsize
		brsh	oled_draw_vline_end		; --> abort
		cp		YL,XH					; Ystart >= Display_Ysize
		brsh	oled_draw_vline_end		; --> abort
		mov		r0,YL					; make copy of Ystart
		add		r0,YH					; (Ystart + length) > Display_Ysize ?
		cp		XH,r0					; Display_Ysize < (Ystart + length)
		brsh	oled_draw_vline_calc
		sub		XH,YL
		dec		XH						; length = Display_Ysize - Ystart -1
		mov		YH,XH					; copy lenght		
oled_draw_vline_calc:
		add		YH,YL					; Yend = (Ystart + lenght -1)
		dec		YH						; YH = Yend
		mov		r18,XH					; r18 = length
		mov		XH,XL					; Ystart = Yend
		cbi		((SPI0_CS_VPORT)+1),(SPI0_CS)					; VPORT.OUT.CS = 0		--> SPI DEVICE SELECTED
		rcall	oled_set_xy
;SPI write Command
		cbi		((SSD1351_DC_VPORT)+1),(SSD1351_DC)				; VPORT.OUT.DC = 0		--> (COMMAND)
		ldi		XH,(SSD1351_CMD_WRITERAM)						;  write COMMAND 
		sts		(SPI0_DATA),XH									; "WRITE DATA TO DRAM"
;wait spi->TX->ready
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_TXCIF_bp)
		rjmp	PC -3
		sts		(SPI0_INTFLAGS),r0
		sbi		((SSD1351_DC_VPORT)+1),(SSD1351_DC)				; VPORT.OUT.DC = 1		--> (DATA)
oled_draw_vline_loop:
;SPI write Data									
		sts		(SPI0_DATA),r17								; write DATA PIX COLOR H
;SPI write Data									
		sts		(SPI0_DATA),r16								; write DATA PIX COLOR L
;wait spi->TX->ready
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_TXCIF_bp)
		rjmp	PC -3
		sts		(SPI0_INTFLAGS),r0
		dec		r18
		brne	oled_draw_vline_loop
		sbi		((SPI0_CS_VPORT)+1),(SPI0_CS)					; VPORT.OUT.CS = 1		--> SPI DEVICE DESELECTED
oled_draw_vline_end:
		ret
#endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef	use_draw_line
oled_draw_line:			; XL = Xstart	|	XH = Xend	|	YL = Ystart	|	YH = Yend	|	r17:r16 = COLOR
		sbrc	XL,7
		ldi		XL,((display_Xsize)-1)
		sbrc	XH,7
		ldi		XH,((display_Xsize)-1)
		sbrc	YL,7
		ldi		YL,((display_Ysize)-1)
		sbrc	YH,7
		ldi		YH,((display_Ysize)-1)
		movw	r2,XL					; copy	Xstart | Xend
		movw	r4,YL					; copy	Ystart | Yend
		cp		r2,r3					; (x0 == x1)
		brne	oled_draw_line_b		; -------------> (x0 != x1)
		cp		r5,r4					; if (x0 == x1)  -->	(y0 > y1)? -> (y1 <= y0)?
		brsh	oled_draw_line_a		; -> (y1 >= y0)?
		mov		YL,r5
		mov		YH,r4					; swap(Ystart,Yend)
oled_draw_line_a:
		sub		YH,YL
		inc		YH						; Ylenght = Yend - Ystart
		rjmp	oled_draw_vline
oled_draw_line_b:						; -------------> (x0 != x1)
		cp		r4,r5					; (y0 == y1)
		brne	oled_write_line			; -------------> (y0 != y1)
		cp		r3,r2					; if (y0 == y1)  -->	(x0 > x1)? -> (x1 <= x0)?
		brsh	oled_draw_line_c		; -> (x1 >= x0)?
		mov		XL,r3
		mov		XH,r2					; swap(Xstart,Xend)
oled_draw_line_c:
		sub		XH,XL
		inc		XH						; Xlenght = Xend - Xstart
		rjmp	oled_draw_hline

;***************************************************************************************************************************************
oled_write_line:
; bool steep = abs(y1 - y0) > abs(x1 - x0)
		mov		r6,r3
		sub		r6,r2					; r6 = (x1 - x0)
		sbrc	r6,7
		neg		r6						; r6 = abs(x1 - x0)
		mov		r7,r5
		sub		r7,r4					; r7 = (y1 - y0)
		sbrc	r7,7
		neg		r7						; r7 = abs(y1 - y0)
		clr		r12						; clr --> steep = 0	(bool)
		cp		r6,r7					; wenn (abs(x1 - x0) >= abs(y1 - y0)) -----> ueberspringe folgende Anweisungsliste
		brsh	oled_write_line_aa
; if (steep)
		mov		r12,ONE					; set --> steep = 1 (bool)
; swap(x0, y0)
		mov		r8,r4					; (temp) r8 <-- r4 (y0)
		mov		r4,r2					;	(y0) r4 <-- r2 (x0)
		mov		r2,r8					;	(x0) r2 <-- r8 (temp/y0)
; swap(x1, y1)
		mov		r8,r5					; (temp) r8 <-- r5 (y1)
		mov		r5,r3					;	(y1) r5 <-- r3 (x1)
		mov		r3,r8					;	(x1) r3 <-- r8 (temp/y1)
oled_write_line_aa:
; if (x0 > x1)
		cp		r3,r2					; wenn  (x1 >= x0) -----> ueberspringe folgende Anweisungsliste
		brsh	oled_write_line_bb
; swap(x0, x1)
		mov		r8,r3					; (temp) r8 <-- r3 (x1)
		mov		r3,r2					;	(x1) r3 <-- r2 (x0)
		mov		r2,r8					;	(x0) r2 <-- r8 (temp/x1)
; swap(y0, y1)
		mov		r8,r5					; (temp) r8 <-- r5 (y1)
		mov		r5,r4					;	(y1) r5 <-- r4 (y0)
		mov		r4,r8					;	(y0) r4 <-- r8 (temp/y1)
oled_write_line_bb:
; dx = x1 - x0
		mov		r6,r3					; r6 = dx
		sub		r6,r2					; r6 = (x1 - x0)
; dy = abs(y1 - y0)
		mov		r7,r5					; r7 = dy
		sub		r7,r4					; r7 = (y1 - y0)
		sbrc	r7,7
		neg		r7						; r7 = abs(y1 - y0)
; err = dx / 2
		mov		r10,r6					; r10 = err(lo)
		clr		r11						; r11 = err(hi)
		lsr		r10						; r10 = dx/2
; if(y0 < y1)
		cp		r4,r5					; wenn (y0 >= y1) -----> ueberspringe folgende Anweisungsliste
		brsh	oled_write_line_cc
; ystep = 1
		mov		r9,ONE					; r9 = ystep = 1
		rjmp	oled_write_line_loop
oled_write_line_cc:
; ystep = -1
		clr		r9
		dec		r9						; r9 = ystep = -1
oled_write_line_loop:
; if(; x1 < x0) ----> break 
		cp		r3,r2
		brlo	oled_write_line_end
; if(steep)
		sbrs	r12,0
		rjmp	oled_write_line_loop_else
; Draw_Pixel(y0, x0)
		mov		XL,r4
		mov		YL,r2
		rcall	oled_draw_pixel
		rjmp	oled_write_line_loop_err
oled_write_line_loop_else:
; Draw_Pixel(x0, y0)
		mov		XL,r2
		mov		YL,r4
		rcall	oled_draw_pixel
oled_write_line_loop_err:
; err -= dy
		sub		r10,r7
		sbc		r11,ZERO
; if(err < 0)
		cp		ZERO,r10
		cpc		ZERO,r11				; wenn if(0 >= err) -----> ueberspringe folgende Anweisungsliste
		brlt	oled_write_line_loop_next
; y0 += ystep
		add		r4,r9
; err += dx
		add		r10,r6
		adc		r11,ZERO
oled_write_line_loop_next:
; x0++
		add		r2,ONE
		rjmp	oled_write_line_loop
oled_write_line_end:
		ret
#endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef	use_draw_circle
oled_draw_circle:		; oled_draw_circle(xc,yc,r,color)			(XL = xc // YL = yc // XH = r // r17:r16 = color)
		movw	r2,XL
		movw	r4,YL
		mov		r5,ONE				; f = 1 - r
		clr		r11
		sub		r5,r3				; r5 = f
		sbc		r11,ZERO
		mov		r10,ONE				; r10 = ddF_x = 1
		ldi		YH,(-2)
		muls	YH,XH				; ddF_y = -2 * r
		movw	r6,r0				; r6 = ddF_y
		clr		r8					; r8 = x = 0
		mov		r9,r3				; r9 = y = r
;----------------------------------------------------------------						
		mov		YL,r4				
		add		YL,r9				; y = y+r
		rcall	oled_draw_pixel
		mov		XL,r2
		mov		YL,r4
		sub		YL,r9				; y = y-r
		rcall	oled_draw_pixel
		mov		XL,r2
		mov		YL,r4
		add		XL,r9				; x = x+r
		rcall	oled_draw_pixel
		mov		XL,r2
		mov		YL,r4
		sub		XL,r9				; x = x-r
		rcall	oled_draw_pixel
;----------------------------------------------------------------
oled_draw_circle_while:				; x < y
		ldi		YH,2				; YH = 2
		cp		r5,ZERO
		cpc		r11,ZERO			; if ( f >= 0) ?
		brlt	oled_draw_circle_while_a		; branch if False
		dec		r9					; y = y-1
		add		r6,YH				; ddF_y = ddF_y + 2
		adc		r7,ZERO				; ddF_y = ddF_y + 2
		add		r5,r6				; f = f + ddF_y
		adc		r11,r7				; f = f + ddF_y
oled_draw_circle_while_a:
		inc		r8					; x = x+1
		add		r10,YH				; ddF_x = ddF_x + 2
		add		r5,r10				; f = f + ddF_x
		adc		r11,ZERO			; f = f + ddF_x
;----------------------------------------------------------------
		mov		XL,r2				; draw_pixel(x0 + x, y0 + y)
		mov		YL,r4
		add		XL,r8				; px = px + x
		add		YL,r9				; py = py + y
		rcall	oled_draw_pixel	
		mov		XL,r2				; draw_pixel(x0 - x, y0 + y)
		mov		YL,r4
		sub		XL,r8				; px = px - x
		add		YL,r9				; py = py + y
		rcall	oled_draw_pixel
		mov		XL,r2				; draw_pixel(x0 + x, y0 - y)
		mov		YL,r4
		add		XL,r8				; px = px + x
		sub		YL,r9				; py = py - y
		rcall	oled_draw_pixel
		mov		XL,r2				; draw_pixel(x0 - x, y0 - y)
		mov		YL,r4
		sub		XL,r8				; px = px - x
		sub		YL,r9				; py = py - y
		rcall	oled_draw_pixel
		mov		XL,r2				; draw_pixel(x0 + y, y0 + x)
		mov		YL,r4
		add		XL,r9				; px = px + y
		add		YL,r8				; py = py + x
		rcall	oled_draw_pixel
		mov		XL,r2				; draw_pixel(x0 - y, y0 + x)
		mov		YL,r4
		sub		XL,r9				; px = px - y
		add		YL,r8				; py = py + x
		rcall	oled_draw_pixel
		mov		XL,r2				; draw_pixel(x0 + y, y0 - x)
		mov		YL,r4
		add		XL,r9				; px = px + y
		sub		YL,r8				; py = py - x
		rcall	oled_draw_pixel
		mov		XL,r2				; draw_pixel(x0 - y, y0 - x)
		mov		YL,r4
		sub		XL,r9				; px = px - y
		sub		YL,r8				; py = py - x
		rcall	oled_draw_pixel
		cp		r8,r9
		brlt	oled_draw_circle_while
		ret
#endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef	use_draw_fill_circle
oled_draw_fill_circle:
		movw	r2,XL
		movw	r4,YL
		sub		YL,r3
		mov		YH,r3
		lsl		YH
		add		YH,ONE
		rcall	oled_draw_vline		; Draw_FastVLine(x0, y0-r, 2*r+1)
		movw	XL,r2
		ldi		YH,3				; corner = 3
		mov		r12,YH				; r12 = corner
;------------------------------------------------------------------------
oled_fill_circle_helper:	
		mov		r5,ONE				; f = 1 - r
		clr		r11
		sub		r5,r3				; r5 = f
		sbc		r11,ZERO
		mov		r10,ONE				; r10 = ddF_x = 1
		ldi		YH,(-2)
		muls	YH,XH				; ddF_y = -2 * r
		movw	r6,r0				; r6 = ddF_y
		clr		r8					; r8 = x = 0
		mov		r9,r3				; r9 = y = r
;----------------------------------------------------------------
oled_fill_circle_helper_while:		; x < y
		ldi		YH,2				; YH = 2
		cp		r5,ZERO
		cpc		r11,ZERO			; if ( f >= 0) ?
		brlt	oled_fill_circle_helper_while_a		; branch if False
		dec		r9					; y = y-1
		add		r6,YH				; ddF_y = ddF_y + 2
		adc		r7,ZERO				; ddF_y = ddF_y + 2
		add		r5,r6				; f = f + ddF_y
		adc		r11,r7				; f = f + ddF_y
oled_fill_circle_helper_while_a:
		inc		r8					; x = x+1
		add		r10,YH				; ddF_x = ddF_x + 2
		add		r5,r10				; f = f + ddF_x
		adc		r11,ZERO			; f = f + ddF_x
;----------------------------------------------------------------
oled_fill_circle_helper_while_b:			; if(corner & 0x1)
		sbrs	r12,0
		rjmp	oled_fill_circle_helper_while_c
		mov		XL,r2
		mov		YL,r4
		add		XL,r8
		sub		YL,r9
		mov		YH,r9
		lsl		YH
		inc		YH
		rcall	oled_draw_vline		; Draw_FastVLine(x0+x, y0-y, 2*y+1+delta)		
		mov		XL,r2
		mov		YL,r4
		add		XL,r9
		sub		YL,r8
		mov		YH,r8
		lsl		YH
		inc		YH
		rcall	oled_draw_vline		; Draw_FastVLine(x0+y, y0-x, 2*x+1+delta)
;----------------------------------------------------------------
oled_fill_circle_helper_while_c:			; if(corner & 0x2)
		sbrs	r12,1
		rjmp	oled_fill_circle_helper_while_d
		mov		XL,r2
		mov		YL,r4
		sub		XL,r8
		sub		YL,r9
		mov		YH,r9
		lsl		YH
		inc		YH
		rcall	oled_draw_vline		; Draw_FastVLine(x0-x, y0-y, 2*y+1+delta)
		mov		XL,r2
		mov		YL,r4
		sub		XL,r9
		sub		YL,r8
		mov		YH,r8
		lsl		YH
		inc		YH
		rcall	oled_draw_vline		; Draw_FastVLine(x0-y, y0-x, 2*x+1+delta)	
;----------------------------------------------------------------
oled_fill_circle_helper_while_d:
		cp		r8,r9
		brlt	oled_fill_circle_helper_while
		ret
#endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef	use_draw_fill_rect
oled_draw_fill_rect:					; XL = Xstart	|	YL = Ystart	|	XH = lenght|	YH = lenght	|	r17:r16 = COLOR
		cpi		XL,(Display_Xsize)		; Xstart >= Display_Xsize
		brsh	oled_fill_rect_end		; --> abort
		cpi		YL,(Display_Ysize)		; Ystart >= Display_Ysize
		brsh	oled_fill_rect_end		; --> abort
		movw	r2,XL					; make copy of X
		movw	r4,YL					; make copy of Y
// Y bounds check
oled_fill_rect_a:
		add		r4,YH
		ldi		YH,(Display_Ysize)
		cp		YH,r4
		brsh	oled_fill_rect_b
		sub		YH,YL					; -------------> ys + yw > (Display_Ysize)
		dec		YH						; yw = (Display_Ysize) - y -1
		movw	r4,YL					; make new copy of Y
// X bounds check
oled_fill_rect_b:						; -------------> ys + yw <= (Display_Ysize)
		mov		YH,r5
		mov		r4,YL
		add		r2,XH
		ldi		XH,(Display_Xsize)
		cp		XH,r2
		brsh	oled_fill_rect_c
		sub		XH,XL					; -------------> xs + xw > (Display_Xsize)
		dec		XH						; xw = (Display_Xsize) - x -1
		movw	r2,XL					; make new copy of X
// set location
oled_fill_rect_c:
		mov		XH,r3
		mov		r2,XL
		add		XH,r2
		dec		XH
		add		YH,r4
		dec		YH
		cbi		((SPI0_CS_VPORT)+1),(SPI0_CS)					; VPORT.OUT.CS = 0		--> SPI DEVICE SELECTED
		rcall	oled_set_xy
;SPI write Command
		mov		r1,r18
		cbi		((SSD1351_DC_VPORT)+1),(SSD1351_DC)				; VPORT.OUT.DC = 0		--> (COMMAND)
		ldi		r18,(SSD1351_CMD_WRITERAM)						;  write COMMAND 
		sts		(SPI0_DATA),r18									; "WRITE DATA TO DRAM"
;wait spi->TX->ready
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_TXCIF_bp)
		rjmp	PC -3
		sts		(SPI0_INTFLAGS),r0
		sbi		((SSD1351_DC_VPORT)+1),(SSD1351_DC)				; VPORT.OUT.DC = 1		--> (DATA)
		mul		r3,r5
		movw	XL,r0
oled_fill_rect_loop:		
;SPI write Data									
		sts		(SPI0_DATA),r17								; write DATA PIX COLOR H
;SPI write Data									
		sts		(SPI0_DATA),r16								; write DATA PIX COLOR L
;wait spi->TX->ready
		lds		r0,(SPI0_INTFLAGS)
		sbrs	r0,(SPI_TXCIF_bp)
		rjmp	PC -3
		sts		(SPI0_INTFLAGS),r0
		sbiw	XL,1
		brne	oled_fill_rect_loop
		sbi		((SPI0_CS_VPORT)+1),(SPI0_CS)					; VPORT.OUT.CS = 1		--> SPI DEVICE DESELECTED
		mov		r18,r1
oled_fill_rect_end:		
		ret
#endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef	use_draw_rect
oled_draw_rect:					; XL = Xstart	|	YL = Ystart	|	XH = widht	|	YH = hight	|	r17:r16 = COLOR
		movw	r2,XL
		movw	r4,YL
		rcall	oled_draw_hline			; Draw_FastHLine(x, y, w)				; TOP
		movw	XL,r2
		movw	YL,r4
		rcall	oled_draw_vline			; Draw_FastVLine(x, y, h)				; LEFT
		movw	XL,r2
		movw	YL,r4
		add		YL,YH
		dec		YL
		rcall	oled_draw_hline			; Draw_FastHLine(x, y+h-1, w)			; RIGHT
		movw	XL,r2
		movw	YL,r4
		add		XL,XH
		dec		XL
		rcall	oled_draw_vline			; Draw_FastVLine(x+w-1, y, h)			; BOTTOM
		ret
#endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef	use_draw_round_rect
oled_draw_round_rect:			; XL = Xstart	|	YL = Ystart	|	XH = widht	|	YH = hight	|	r18 = radius	|	r17:r16 = COLOR
		push	r19
		push	r20
		push	r21
		push	r22
		push	r23
		movw	r2,XL
		movw	r4,YL
		movw	r20,r2
		movw	r22,r4	
		mov		r12,r18
		mov		r19,r18
		add		XL,r19
		lsl		r12
		sub		XH,r12		
		rcall	oled_draw_hline			; Draw_FastHLine(x+r  , y    , w-2*r)		// Top
		movw	XL,r2
		movw	YL,r4
		add		XL,r19
		add		YL,YH
		dec		YL
		sub		XH,r12
		rcall	oled_draw_hline			; Draw_FastHLine(x+r  , y+h-1, w-2*r)		// Bottom
		movw	XL,r2
		movw	YL,r4
		add		YL,r19
		sub		YH,r12
		rcall	oled_draw_vline			; Draw_FastVLine(x    , y+r  , h-2*r)		// Left
		movw	XL,r2
		movw	YL,r4
		add		XL,XH
		dec		XL
		add		YL,r19
		sub		YH,r12
		rcall	oled_draw_vline			; Draw_FastVLine(x+w-1, y+r  , h-2*r)		// Right
		mov		r18,r19

		movw	XL,r20
		movw	YL,r22	
;--------------------------------------		
		add		XL,r18
		add		YL,r18
		ldi		r19,1							; r19 = corner
		rcall	oled_draw_circle_helper			; DrawCircle_Helper(x+r, y+r, r, 1)
;--------------------------------------
		movw	XL,r20
		movw	YL,r22	
;--------------------------------------		
		add		XL,XH
		sub		XL,r18
		dec		XL
		add		YL,r18
		ldi		r19,2							; r19 = corner
		rcall	oled_draw_circle_helper			; DrawCircle_Helper(x+w-r-1, y+r, r, 2)
;--------------------------------------
		movw	XL,r20
		movw	YL,r22	
;--------------------------------------		
		add		XL,XH
		sub		XL,r18
		dec		XL
		add		YL,YH
		sub		YL,r18
		dec		YL
		ldi		r19,4							; r19 = corner
		rcall	oled_draw_circle_helper			; DrawCircle_Helper(x+w-r-1, y+h-r-1, r, 4)
;--------------------------------------
		movw	XL,r20
		movw	YL,r22	
;--------------------------------------		
		add		XL,r18
		add		YL,YH
		sub		YL,r18
		dec		YL
		ldi		r19,8							; r19 = corner
		rcall	oled_draw_circle_helper			; DrawCircle_Helper(x+r, y+h-r-1, r, 8)
;--------------------------------------
		pop		r23
		pop		r22
		pop		r21
		pop		r20
		pop		r19
		ret
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
oled_draw_circle_helper:
		movw	r2,XL
		movw	r4,YL
		mov		r10,ONE				; f = 1 - r
		clr		r11
		sub		r10,r18				; r10 = f
		sbc		r11,ZERO
		mov		r12,ONE				; r12 = ddF_x = 1
		ldi		YH,(-2)
		muls	YH,r18				; ddF_y = -2 * r
		movw	r6,r0				; r6 = ddF_y
		clr		r8					; r8 = x = 0
		mov		r9,r18				; r9 = y = r
;----------------------------------------------------------------
oled_draw_circle_helper_while:		; x < y
		ldi		YH,2				; YH = 2
		cp		r10,ZERO
		cpc		r11,ZERO			; if ( f >= 0) ?
		brlt	oled_draw_circle_helper_while_a		; branch if False
		dec		r9					; y = y-1
		add		r6,YH				; ddF_y = ddF_y + 2
		adc		r7,ZERO				; ddF_y = ddF_y + 2
		add		r10,r6				; f = f + ddF_y
		adc		r11,r7				; f = f + ddF_y
oled_draw_circle_helper_while_a:
		inc		r8					; x = x+1
		add		r12,YH				; ddF_x = ddF_x + 2
		add		r10,r12				; f = f + ddF_x
		adc		r11,ZERO			; f = f + ddF_x
;----------------------------------------------------------------
oled_draw_circle_helper_while_b:			; if(corner & 0x4)
		sbrs	r19,2
		rjmp	oled_draw_circle_helper_while_c
		mov		XL,r2
		mov		YL,r4
		add		XL,r8
		add		YL,r9
		rcall	oled_draw_pixel		;Draw_Pixel(x0 + x, y0 + y)	
		mov		XL,r2
		mov		YL,r4
		add		XL,r9
		add		YL,r8
		rcall	oled_draw_pixel		; Draw_Pixel(x0 + y, y0 + x)
;----------------------------------------------------------------
oled_draw_circle_helper_while_c:			; if(corner & 0x2)
		sbrs	r19,1
		rjmp	oled_draw_circle_helper_while_d
		mov		XL,r2
		mov		YL,r4
		add		XL,r8
		sub		YL,r9
		rcall	oled_draw_pixel		; Draw_Pixel(x0 + x, y0 - y)
		mov		XL,r2
		mov		YL,r4
		add		XL,r9
		sub		YL,r8
		rcall	oled_draw_pixel		; Draw_Pixel(x0 + y, y0 - x)
;----------------------------------------------------------------
oled_draw_circle_helper_while_d:			; if(corner & 0x8)
		sbrs	r19,3
		rjmp	oled_draw_circle_helper_while_e
		mov		XL,r2
		mov		YL,r4
		sub		XL,r9
		add		YL,r8
		rcall	oled_draw_pixel		; Draw_Pixel(x0 - y, y0 + x)
		mov		XL,r2
		mov		YL,r4
		sub		XL,r8
		add		YL,r9
		rcall	oled_draw_pixel		; Draw_Pixel(x0 - x, y0 + y)
;----------------------------------------------------------------
oled_draw_circle_helper_while_e:			; if(corner & 0x1)
		sbrs	r19,0
		rjmp	oled_draw_circle_helper_while_f
		mov		XL,r2
		mov		YL,r4
		sub		XL,r9
		sub		YL,r8
		rcall	oled_draw_pixel		; Draw_Pixel(x0 - y, y0 - x)
		mov		XL,r2
		mov		YL,r4
		sub		XL,r8
		sub		YL,r9
		rcall	oled_draw_pixel		; Draw_Pixel(x0 - x, y0 - y)
;----------------------------------------------------------------
oled_draw_circle_helper_while_f:
		cp		r8,r9
		brlt	oled_draw_circle_helper_while
		ret
#endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef	use_draw_fill_round_rect
oled_draw_fill_round_rect:			; XL = Xstart	|	YL = Ystart	|	XH = widht	|	YH = hight	|	r18 = radius	|	r17:r16 = COLOR
		push	r19
		push	r20
		push	r21
		push	r22
		push	r23
		movw	r2,XL
		movw	r4,YL
		movw	r20,r2
		movw	r22,r4
		mov		r19,r18	
		add		r2,r19					; r2 = (x+r)
		mov		r6,r19					; r6 = r
		lsl		r6						; r6 = 2*r
		sub		r3,r6					; r6 = (w-2*r)
		mov		r6,r19					; r6 = i = r
;--------------------------------------------------------------------------------------
oled_draw_fill_round_rect_loop_a:
		mov		XL,r2
		mov		YL,r4
		dec		YL
		add		YL,r6
		mov		XH,r3	
		rcall	oled_draw_hline			; Draw_FastHLine(x+r , y+i , w-2*r)		// Top
		dec		r6
		brne	oled_draw_fill_round_rect_loop_a
;--------------------------------------------------------------------------------------
;rjmp	test
		mov		r6,r23					; r6 = h
		mov		r5,r22					; r5 = y
		add		r5,r6					; r5 = y+h
;		dec		r5						; r5 = y+h-1
		mov		r6,r19					; r6 = i = r
;--------------------------------------------------------------------------------------
oled_draw_fill_round_rect_loop_b:
		mov		XL,r2
		mov		YL,r5
		sub		YL,r6
		mov		XH,r3	
		rcall	oled_draw_hline			; Draw_FastHLine(x+r , (y+h-1)-i) , w-2*r)	// Bottom
		dec		r6
		brne	oled_draw_fill_round_rect_loop_b
;--------------------------------------------------------------------------------------
;test:
		movw	XL,r20					
		movw	YL,r22
		movw	r2,XL
		movw	r4,YL
		add		YL,r19
		dec		YL
		mov		r4,YL					; r4 = (y+r)
		mov		r6,r19					; r6 = r
		lsl		r6						; r6 = 2*r
		sub		r5,r6					; r5 = h-2*r  ----->  r5 = i
;--------------------------------------------------------------------------------------	
oled_draw_fill_round_rect_loop_c:
		mov		XL,r2					; for( i=h-2*r ; i > 0 ; i--)
		mov		YL,r4					; YL = y+r
		add		YL,r5					; YL = YL+i
		mov		XH,r3					; XH = xw
		rcall	oled_draw_hline			; Draw_FastHLine(x, y+r+i-1,w)				// Center
		dec		r5
		brne	oled_draw_fill_round_rect_loop_c
;--------------------------------------------------------------------------------------
		mov		r3,r19
		movw	XL,r20
		movw	YL,r22
		mov		r2,XL
		movw	r4,YL	
;--------------------------------------		
		add		XL,r3
		add		YL,r3
		ldi		r19,1									; r19 = corner
		rcall	oled_draw_fill_circle_helper			; DrawCircle_Helper(x+r, y+r, r, 1)
;--------------------------------------
		movw	XL,r20
		movw	YL,r22	
;--------------------------------------		
		add		XL,XH
		sub		XL,r3
;		dec		XL
		add		YL,r3
		ldi		r19,2									; r19 = corner
		rcall	oled_draw_fill_circle_helper			; DrawCircle_Helper(x+w-r-1, y+r, r, 2)
;--------------------------------------
		movw	XL,r20
		movw	YL,r22	
;--------------------------------------		
		add		XL,XH
		sub		XL,r3
;		dec		XL
		add		YL,YH
		sub		YL,r3
		dec		YL
		ldi		r19,4									; r19 = corner
		rcall	oled_draw_fill_circle_helper			; DrawCircle_Helper(x+w-r-1, y+h-r-1, r, 4)
;--------------------------------------
		movw	XL,r20
		movw	YL,r22
;--------------------------------------		
		add		XL,r3
		add		YL,YH
		sub		YL,r3
		dec		YL
		ldi		r19,8									; r19 = corner
		rcall	oled_draw_fill_circle_helper			; DrawCircle_Helper(x+r, y+h-r-1, r, 8)
;--------------------------------------
		pop		r23
		pop		r22
		pop		r21
		pop		r20
		pop		r19
		ret

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
oled_draw_fill_circle_helper:
		mov		r18,r3
		mov		r2,XL
		movw	r4,YL
		mov		r10,ONE				; f = 1 - r
		clr		r11
		sub		r10,r18				; r10 = f
		sbc		r11,ZERO
		mov		r12,ONE				; r12 = ddF_x = 1
		ldi		YH,(-2)
		muls	YH,r18				; ddF_y = -2 * r
		movw	r6,r0				; r6 = ddF_y
		clr		r8					; r8 = x = 0
		mov		r9,r18				; r9 = y = r
;----------------------------------------------------------------
oled_draw_fill_circle_helper_while:		; x < y
		ldi		YH,2				; YH = 2
		cp		r10,ZERO
		cpc		r11,ZERO			; if ( f >= 0) ?
		brlt	oled_draw_fill_circle_helper_while_a		; branch if False
		dec		r9					; y = y-1
		add		r6,YH				; ddF_y = ddF_y + 2
		adc		r7,ZERO				; ddF_y = ddF_y + 2
		add		r10,r6				; f = f + ddF_y
		adc		r11,r7				; f = f + ddF_y
oled_draw_fill_circle_helper_while_a:
		inc		r8					; x = x+1
		add		r12,YH				; ddF_x = ddF_x + 2
		add		r10,r12				; f = f + ddF_x
		adc		r11,ZERO			; f = f + ddF_x
;----------------------------------------------------------------
oled_draw_fill_circle_helper_while_b:			; if(corner & 0x4)
		sbrs	r19,2
		rjmp	oled_draw_fill_circle_helper_while_c
		mov		XL,r2
		mov		YL,r4
		add		YL,r9
		mov		XH,r8
		rcall	oled_draw_hline		;Draw_fast_Hline(x0 , y0 + y , x)	
		mov		XL,r2
		mov		YL,r4
		add		YL,r8
		mov		XH,r9
		rcall	oled_draw_hline		; Draw_fast_Hline(x0 , y0 + x, y)
;----------------------------------------------------------------
oled_draw_fill_circle_helper_while_c:			; if(corner & 0x2)
		sbrs	r19,1
		rjmp	oled_draw_fill_circle_helper_while_d
		mov		XL,r2
		mov		YL,r4
		sub		YL,r9
		mov		XH,r8
		rcall	oled_draw_hline		; Draw_fast_Hline(x0 , y0 - y , x)
		mov		XL,r2
		mov		YL,r4
		sub		YL,r8
		mov		XH,r9
		rcall	oled_draw_hline		; Draw_fast_Hline(x0 , y0 - x , y)
;----------------------------------------------------------------
oled_draw_fill_circle_helper_while_d:			; if(corner & 0x8)
		sbrs	r19,3
		rjmp	oled_draw_fill_circle_helper_while_e
		mov		XL,r2
		mov		YL,r4
		sub		XL,r9
		add		YL,r8
		mov		XH,r9
		rcall	oled_draw_hline		; Draw_fast_Hline(x0 - y, y0 + x , y)
		mov		XL,r2
		mov		YL,r4
		sub		XL,r8
		add		YL,r9
		mov		XH,r8
		rcall	oled_draw_hline		; Draw_fast_Hline(x0 - x, y0 + y, x)
;----------------------------------------------------------------
oled_draw_fill_circle_helper_while_e:			; if(corner & 0x1)
		sbrs	r19,0
		rjmp	oled_draw_fill_circle_helper_while_f
		mov		XL,r2
		mov		YL,r4
		sub		XL,r9
		sub		YL,r8
		mov		XH,r9
		rcall	oled_draw_hline		; Draw_fast_Hline(x0 - y, y0 - x , y)
		mov		XL,r2
		mov		YL,r4
		sub		XL,r8
		sub		YL,r9
		mov		XH,r8
		rcall	oled_draw_hline		; Draw_fast_Hline(x0 - x , y0 - y , x)
;----------------------------------------------------------------
oled_draw_fill_circle_helper_while_f:
		cp		r8,r9
		brlt	jmp_oled_draw_fill_circle_helper_while
		ret

jmp_oled_draw_fill_circle_helper_while:
		rjmp	oled_draw_fill_circle_helper_while
#endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
oled_draw_triangel:

		ret

oled_draw_fill_triangel:

		ret
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
.macro  draw_hline						; draw_hline	x,y,leght,color		// draw horizontal line
		ldi		XL,@0
		ldi		YL,@1
        ldi		XH,@2
		ldi		r16,LOW(@3)
		ldi		r17,HIGH(@3)
		rcall	oled_draw_hline
.endmacro

.macro  draw_vline						; draw_vline	x,y,leght,color		// draw vertical line
		ldi		XL,@0
		ldi		YL,@1
        ldi		XH,@2
		ldi		r16,LOW(@3)
		ldi		r17,HIGH(@3)
		rcall	oled_draw_vline
.endmacro

.macro  draw_line						; draw_line		xs,ys,xe,ye,color		// draw line
		ldi		XL,@0
		ldi		YL,@1
        ldi		XH,@2
		ldi		YH,@3
		ldi		r16,LOW(@4)
		ldi		r17,HIGH(@4)
		rcall	oled_draw_line
.endmacro

.macro  draw_rect						; draw_rect		x,y,xw,yh,color		// draw rect
		ldi		XL,@0
		ldi		YL,@1
        ldi		XH,@2
		ldi		YH,@3
		ldi		r16,LOW(@4)
		ldi		r17,HIGH(@4)
		rcall	oled_draw_rect
.endmacro

.macro  draw_frect						; draw_frect		x,y,xw,yh,color		// draw filled rect
		ldi		XL,@0
		ldi		YL,@1
        ldi		XH,@2
		ldi		YH,@3
		ldi		r16,LOW(@4)
		ldi		r17,HIGH(@4)
		rcall	oled_draw_fill_rect
.endmacro

.macro  draw_rdrect					; draw_rdrect		x,y,xw,yh,r,color		// draw rounded rect
		ldi		XL,@0
		ldi		YL,@1
        ldi		XH,@2
		ldi		YH,@3
		ldi		r18,@4
		ldi		r16,LOW(@5)
		ldi		r17,HIGH(@5)
		rcall	oled_draw_round_rect
.endmacro

.macro  draw_frdrect					; draw_frdrect		x,y,xw,yh,r,color		// draw filled rounded rect
		ldi		XL,@0
		ldi		YL,@1
        ldi		XH,@2
		ldi		YH,@3
		ldi		r18,@4
		ldi		r16,LOW(@5)
		ldi		r17,HIGH(@5)
		rcall	oled_draw_fill_round_rect
.endmacro

.macro  draw_circle						; draw_circle		x,y,r,color			// draw circle
		ldi		XL,@0
		ldi		YL,@1
        ldi		XH,@2
		ldi		r16,LOW(@3)
		ldi		r17,HIGH(@3)
		rcall	oled_draw_circle
.endmacro

.macro  draw_fcircle					; draw_fcircle		x,y,r,color			// draw filled circle
		ldi		XL,@0
		ldi		YL,@1
        ldi		XH,@2
		ldi		r16,LOW(@3)
		ldi		r17,HIGH(@3)
		rcall	oled_draw_fill_circle
.endmacro

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;--- Macros -------------------------------------------------------

.macro  load_p                      ; ldt 16Bit @1 nach @0 High und @0 Low
        ldi     @0H,HIGH(@1)
        ldi     @0L,LOW(@1)
.endmacro

.macro  ldi16                      ; ldt 16Bit @2 nach @1 High und @0 Low	Bsp:	ldi16	r16,r17,(3000)
        ldi     @0,LOW(@2)
        ldi     @1,HIGH(@2)
.endmacro
